@@ -1,13 +1,15 @@
 __all__ = ['OfflineImap']
 
 __productname__ = 'OfflineIMAP'
-__version__     = "6.5.6"
-__copyright__   = "Copyright 2002-2013 John Goerzen & contributors"
+__version__     = "6.6.1"
+__revision__    = ""
+__bigversion__  = __version__ + __revision__
+__copyright__   = "Copyright 2002-2015 John Goerzen & contributors"
 __author__      = "John Goerzen"
-__author_email__= "john@complete.org"
+__author_email__= "offlineimap-project@lists.alioth.debian.org"
 __description__ = "Disconnected Universal IMAP Mail Synchronization/Reader Support"
-__license__  = "Licensed under the GNU GPL v2+ (v2 or any later version)"
-__bigcopyright__ = """%(__productname__)s %(__version__)s
+__license__  = "Licensed under the GNU GPL v2 or any later version (with an OpenSSL exception)"
+__bigcopyright__ = """%(__productname__)s %(__bigversion__)s
   %(__license__)s""" % locals()
 __homepage__ = "http://offlineimap.org"
 
