@@ -15,19 +15,25 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-from offlineimap import imaplibutil, imaputil, threadutil, OfflineImapError
-from offlineimap.ui import getglobalui
 from threading import Lock, BoundedSemaphore, Thread, Event, currentThread
-import offlineimap.accounts
 import hmac
 import socket
 import base64
+
+import json
+import urllib
+
 import time
 import errno
 from sys import exc_info
 from socket import gaierror
 from ssl import SSLError, cert_time_to_seconds
 
+from offlineimap import imaplibutil, imaputil, threadutil, OfflineImapError
+import offlineimap.accounts
+from offlineimap.ui import getglobalui
+
+
 try:
     # do we have a recent pykerberos?
     have_gss = False
@@ -45,7 +51,8 @@ class IMAPServer:
 
     Public instance variables are: self.:
      delim The server's folder delimiter. Only valid after acquireconnection()
-     """
+    """
+
     GSS_STATE_STEP = 0
     GSS_STATE_WRAP = 1
     def __init__(self, repos):
@@ -56,16 +63,16 @@ class IMAPServer:
         self.preauth_tunnel = repos.getpreauthtunnel()
         self.transport_tunnel = repos.gettransporttunnel()
         if self.preauth_tunnel and self.transport_tunnel:
-            raise OfflineImapError('%s: ' % repos + \
-              'you must enable precisely one '
-              'type of tunnel (preauth or transport), '
-              'not both', OfflineImapError.ERROR.REPO)
+            raise OfflineImapError('%s: '% repos +
+                'you must enable precisely one '
+                'type of tunnel (preauth or transport), '
+                'not both', OfflineImapError.ERROR.REPO)
         self.tunnel = \
-          self.preauth_tunnel if self.preauth_tunnel \
-          else self.transport_tunnel
+            self.preauth_tunnel if self.preauth_tunnel \
+            else self.transport_tunnel
 
         self.username = \
-          None if self.preauth_tunnel else repos.getuser()
+            None if self.preauth_tunnel else repos.getuser()
         self.user_identity = repos.get_remote_identity()
         self.authmechs = repos.get_auth_mechanisms()
         self.password = None
@@ -74,16 +81,24 @@ class IMAPServer:
 
         self.usessl = repos.getssl()
         self.hostname = \
-          None if self.preauth_tunnel else repos.gethost()
+            None if self.preauth_tunnel else repos.gethost()
         self.port = repos.getport()
         if self.port == None:
             self.port = 993 if self.usessl else 143
         self.sslclientcert = repos.getsslclientcert()
         self.sslclientkey = repos.getsslclientkey()
         self.sslcacertfile = repos.getsslcacertfile()
-        self.sslversion = repos.getsslversion()
         if self.sslcacertfile is None:
             self.__verifycert = None # disable cert verification
+        self.fingerprint = repos.get_ssl_fingerprint()
+        self.sslversion = repos.getsslversion()
+        self.tlslevel = repos.gettlslevel()
+
+        self.oauth2_refresh_token = repos.getoauth2_refresh_token()
+        self.oauth2_client_id = repos.getoauth2_client_id()
+        self.oauth2_client_secret = repos.getoauth2_client_secret()
+        self.oauth2_request_url = repos.getoauth2_request_url()
+        self.oauth2_access_token = None
 
         self.delim = None
         self.root = None
@@ -99,6 +114,31 @@ class IMAPServer:
         self.gss_vc = None
         self.gssapi = False
 
+        # In order to support proxy connection, we have to override the
+        # default socket instance with our own socksified socket instance.
+        # We add this option to bypass the GFW in China.
+        _account_section = 'Account ' + self.repos.account.name
+        if not self.config.has_option(_account_section, 'proxy'):
+            self.proxied_socket = socket.socket
+        else:
+            proxy = self.config.get(_account_section, 'proxy')
+            # Powered by PySocks.
+            try:
+                import socks
+                proxy_type, host, port = proxy.split(":")
+                port = int(port)
+                socks.setdefaultproxy(getattr(socks, proxy_type), host, port)
+                self.proxied_socket = socks.socksocket
+            except ImportError:
+                self.ui.warn("PySocks not installed, ignoring proxy option.")
+                self.proxied_socket = socket.socket
+            except (AttributeError, ValueError) as e:
+                self.ui.warn("Bad proxy option %s for account %s: %s "
+                    "Ignoring proxy option."%
+                    (proxy, self.repos.account.name, e))
+                self.proxied_socket = socket.socket
+
+
     def __getpassword(self):
         """Returns the server password or None"""
         if self.goodpassword != None: # use cached good one first
@@ -109,15 +149,16 @@ class IMAPServer:
 
         # get 1) configured password first 2) fall back to asking via UI
         self.password = self.repos.getpassword() or \
-                        self.ui.getpass(self.repos.getname(), self.config,
-                                        self.passworderror)
+            self.ui.getpass(self.repos.getname(), self.config,
+                self.passworderror)
         self.passworderror = None
         return self.password
 
     # XXX: is this function used anywhere?
     def getroot(self):
-        """Returns this server's folder root.  Can only be called after one
+        """Returns this server's folder root. Can only be called after one
         or more calls to acquireconnection."""
+
         return self.root
 
 
@@ -126,6 +167,7 @@ class IMAPServer:
 
         :param drop_conn: If True, the connection will be released and
            not be reused. This can be used to indicate broken connections."""
+
         if connection is None: return #noop on bad connection
         self.connectionlock.acquire()
         self.assignedconnections.remove(connection)
@@ -139,25 +181,24 @@ class IMAPServer:
 
     def __md5handler(self, response):
         challenge = response.strip()
-        self.ui.debug('imap', '__md5handler: got challenge %s' % challenge)
+        self.ui.debug('imap', '__md5handler: got challenge %s'% challenge)
 
         passwd = self.__getpassword()
         retval = self.username + ' ' + hmac.new(passwd, challenge).hexdigest()
-        self.ui.debug('imap', '__md5handler: returning %s' % retval)
+        self.ui.debug('imap', '__md5handler: returning %s'% retval)
         return retval
 
     def __loginauth(self, imapobj):
-        """ Basic authentication via LOGIN command """
+        """ Basic authentication via LOGIN command."""
+
         self.ui.debug('imap', 'Attempting IMAP LOGIN authentication')
         imapobj.login(self.username, self.__getpassword())
 
 
     def __plainhandler(self, response):
-        """
-        Implements SASL PLAIN authentication, RFC 4616,
-          http://tools.ietf.org/html/rfc4616
+        """Implements SASL PLAIN authentication, RFC 4616,
+          http://tools.ietf.org/html/rfc4616"""
 
-        """
         authc = self.username
         passwd = self.__getpassword()
         authz = ''
@@ -165,32 +206,59 @@ class IMAPServer:
             authz = self.user_identity
         NULL = u'\x00'
         retval = NULL.join((authz, authc, passwd)).encode('utf-8')
-        self.ui.debug('imap', '__plainhandler: returning %s' % retval)
+        logsafe_retval = NULL.join((authz, authc, "(passwd hidden for log)")).encode('utf-8')
+        self.ui.debug('imap', '__plainhandler: returning %s' % logsafe_retval)
         return retval
 
 
-    # XXX: describe function
+    def __xoauth2handler(self, response):
+        if self.oauth2_refresh_token is None:
+            return None
+
+        if self.oauth2_access_token is None:
+            # need to move these to config
+            # generate new access token
+            params = {}
+            params['client_id'] = self.oauth2_client_id
+            params['client_secret'] = self.oauth2_client_secret
+            params['refresh_token'] = self.oauth2_refresh_token
+            params['grant_type'] = 'refresh_token'
+
+            self.ui.debug('imap', 'xoauth2handler: url "%s"' % self.oauth2_request_url)
+            self.ui.debug('imap', 'xoauth2handler: params "%s"' % params)
+
+            response = urllib.urlopen(self.oauth2_request_url, urllib.urlencode(params)).read()
+            resp = json.loads(response)
+            self.ui.debug('imap', 'xoauth2handler: response "%s"' % resp)
+            self.oauth2_access_token = resp['access_token']
+
+        self.ui.debug('imap', 'xoauth2handler: access_token "%s"' % self.oauth2_access_token)
+        auth_string = 'user=%s\1auth=Bearer %s\1\1' % (self.username, self.oauth2_access_token)
+        #auth_string = base64.b64encode(auth_string)
+        self.ui.debug('imap', 'xoauth2handler: returning "%s"' % auth_string)
+        return auth_string
+
     def __gssauth(self, response):
         data = base64.b64encode(response)
         try:
             if self.gss_step == self.GSS_STATE_STEP:
                 if not self.gss_vc:
-                    rc, self.gss_vc = kerberos.authGSSClientInit('imap@' +
-                                                                 self.hostname)
+                    rc, self.gss_vc = kerberos.authGSSClientInit(
+                        'imap@' + self.hostname)
                     response = kerberos.authGSSClientResponse(self.gss_vc)
                 rc = kerberos.authGSSClientStep(self.gss_vc, data)
                 if rc != kerberos.AUTH_GSS_CONTINUE:
-                   self.gss_step = self.GSS_STATE_WRAP
+                    self.gss_step = self.GSS_STATE_WRAP
             elif self.gss_step == self.GSS_STATE_WRAP:
                 rc = kerberos.authGSSClientUnwrap(self.gss_vc, data)
                 response = kerberos.authGSSClientResponse(self.gss_vc)
-                rc = kerberos.authGSSClientWrap(self.gss_vc, response,
-                                                self.username)
+                rc = kerberos.authGSSClientWrap(
+                    self.gss_vc, response, self.username)
             response = kerberos.authGSSClientResponse(self.gss_vc)
         except kerberos.GSSError as err:
             # Kerberos errored out on us, respond with None to cancel the
             # authentication
-            self.ui.debug('imap', '%s: %s' % (err[0][0], err[1][0]))
+            self.ui.debug('imap', '%s: %s'% (err[0][0], err[1][0]))
             return None
 
         if not response:
@@ -205,8 +273,8 @@ class IMAPServer:
                 imapobj.starttls()
             except imapobj.error as e:
                 raise OfflineImapError("Failed to start "
-                  "TLS connection: %s" % str(e),
-                  OfflineImapError.ERROR.REPO)
+                    "TLS connection: %s"% str(e),
+                    OfflineImapError.ERROR.REPO, None, exc_info()[2])
 
 
     ## All __authn_* procedures are helpers that do authentication.
@@ -253,21 +321,24 @@ class IMAPServer:
         imapobj.authenticate('PLAIN', self.__plainhandler)
         return True
 
+    def __authn_xoauth2(self, imapobj):
+        imapobj.authenticate('XOAUTH2', self.__xoauth2handler)
+        return True
+
     def __authn_login(self, imapobj):
         # Use LOGIN command, unless LOGINDISABLED is advertized
         # (per RFC 2595)
         if 'LOGINDISABLED' in imapobj.capabilities:
             raise OfflineImapError("IMAP LOGIN is "
-              "disabled by server.  Need to use SSL?",
-               OfflineImapError.ERROR.REPO)
+                "disabled by server.  Need to use SSL?",
+                OfflineImapError.ERROR.REPO)
         else:
             self.__loginauth(imapobj)
             return True
 
 
     def __authn_helper(self, imapobj):
-        """
-        Authentication machinery for self.acquireconnection().
+        """Authentication machinery for self.acquireconnection().
 
         Raises OfflineImapError() of type ERROR.REPO when
         there are either fatal problems or no authentications
@@ -275,9 +346,7 @@ class IMAPServer:
 
         If any authentication method succeeds, routine should exit:
         warnings for failed methods are to be produced in the
-        respective except blocks.
-
-        """
+        respective except blocks."""
 
         # Authentication routines, hash keyed by method name
         # with value that is a tuple with
@@ -287,6 +356,7 @@ class IMAPServer:
         auth_methods = {
           "GSSAPI": (self.__authn_gssapi, False, True),
           "CRAM-MD5": (self.__authn_cram_md5, True, True),
+          "XOAUTH2": (self.__authn_xoauth2, True, True),
           "PLAIN": (self.__authn_plain, True, True),
           "LOGIN": (self.__authn_login, True, False),
         }
@@ -321,13 +391,13 @@ class IMAPServer:
                     continue
 
             tried_to_authn = True
-            self.ui.debug('imap', 'Attempting '
-              '%s authentication' % m)
+            self.ui.debug('imap', u'Attempting '
+              '%s authentication'% m)
             try:
                 if func(imapobj):
                     return
             except (imapobj.error, OfflineImapError) as e:
-                self.ui.warn('%s authentication failed: %s' % (m, e))
+                self.ui.warn('%s authentication failed: %s'% (m, e))
                 exc_stack.append((m, e))
 
         if len(exc_stack):
@@ -336,16 +406,16 @@ class IMAPServer:
               exc_stack
             ))
             raise OfflineImapError("All authentication types "
-              "failed:\n\t%s" % msg, OfflineImapError.ERROR.REPO)
+              "failed:\n\t%s"% msg, OfflineImapError.ERROR.REPO)
 
         if not tried_to_authn:
             methods = ", ".join(map(
               lambda x: x[5:], filter(lambda x: x[0:5] == "AUTH=",
                imapobj.capabilities)
             ))
-            raise OfflineImapError("Repository %s: no supported "
+            raise OfflineImapError(u"Repository %s: no supported "
               "authentication mechanisms found; configured %s, "
-              "server advertises %s" % (self.repos,
+              "server advertises %s"% (self.repos,
               ", ".join(self.authmechs), methods),
               OfflineImapError.ERROR.REPO)
 
@@ -383,35 +453,42 @@ class IMAPServer:
 
         self.connectionlock.release()   # Release until need to modify data
 
-        """ Must be careful here that if we fail we should bail out gracefully
-        and release locks / threads so that the next attempt can try...
-        """
+        # Must be careful here that if we fail we should bail out gracefully
+        # and release locks / threads so that the next attempt can try...
         success = 0
         try:
             while not success:
                 # Generate a new connection.
                 if self.tunnel:
                     self.ui.connecting('tunnel', self.tunnel)
-                    imapobj = imaplibutil.IMAP4_Tunnel(self.tunnel,
-                                                       timeout=socket.getdefaulttimeout())
+                    imapobj = imaplibutil.IMAP4_Tunnel(
+                        self.tunnel,
+                        timeout=socket.getdefaulttimeout(),
+                        use_socket=self.proxied_socket,
+                        )
                     success = 1
                 elif self.usessl:
                     self.ui.connecting(self.hostname, self.port)
-                    fingerprint = self.repos.get_ssl_fingerprint()
-                    imapobj = imaplibutil.WrappedIMAP4_SSL(self.hostname,
-                                                           self.port,
-                                                           self.sslclientkey,
-                                                           self.sslclientcert,
-                                                           self.sslcacertfile,
-                                                           self.__verifycert,
-                                                           self.sslversion,
-                                                           timeout=socket.getdefaulttimeout(),
-                                                           fingerprint=fingerprint
-                                                           )
+                    imapobj = imaplibutil.WrappedIMAP4_SSL(
+                        self.hostname,
+                        self.port,
+                        self.sslclientkey,
+                        self.sslclientcert,
+                        self.sslcacertfile,
+                        self.__verifycert,
+                        self.sslversion,
+                        timeout=socket.getdefaulttimeout(),
+                        fingerprint=self.fingerprint,
+                        use_socket=self.proxied_socket,
+                        tls_level=self.tlslevel,
+                        )
                 else:
                     self.ui.connecting(self.hostname, self.port)
-                    imapobj = imaplibutil.WrappedIMAP4(self.hostname, self.port,
-                                                       timeout=socket.getdefaulttimeout())
+                    imapobj = imaplibutil.WrappedIMAP4(
+                        self.hostname, self.port,
+                        timeout=socket.getdefaulttimeout(),
+                        use_socket=self.proxied_socket,
+                        )
 
                 if not self.preauth_tunnel:
                     try:
@@ -441,12 +518,12 @@ class IMAPServer:
                     # No Folders were returned. This occurs, e.g. if the
                     # 'reference' prefix does not exist on the mail
                     # server. Raise exception.
-                    err = "Server '%s' returned no folders in '%s'" % \
+                    err = "Server '%s' returned no folders in '%s'"% \
                         (self.repos.getname(), self.reference)
                     self.ui.warn(err)
                     raise Exception(err)
                 self.delim, self.root = \
-                            imaputil.imapsplit(listres[0])[1:]
+                     imaputil.imapsplit(listres[0])[1:]
                 self.delim = imaputil.dequote(self.delim)
                 self.root = imaputil.dequote(self.root)
 
@@ -458,6 +535,7 @@ class IMAPServer:
             """If we are here then we did not succeed in getting a
             connection - we should clean up and then re-raise the
             error..."""
+
             self.semaphore.release()
 
             severity = OfflineImapError.ERROR.REPO
@@ -468,20 +546,20 @@ class IMAPServer:
                          "'%s'. Make sure you have configured the ser"\
                          "ver name correctly and that you are online."%\
                          (self.hostname, self.repos)
-                raise OfflineImapError(reason, severity)
+                raise OfflineImapError(reason, severity), None, exc_info()[2]
 
-            elif isinstance(e, SSLError) and e.errno == 1:
+            elif isinstance(e, SSLError) and e.errno == errno.EPERM:
                 # SSL unknown protocol error
                 # happens e.g. when connecting via SSL to a non-SSL service
                 if self.port != 993:
                     reason = "Could not connect via SSL to host '%s' and non-s"\
                         "tandard ssl port %d configured. Make sure you connect"\
-                        " to the correct port." % (self.hostname, self.port)
+                        " to the correct port."% (self.hostname, self.port)
                 else:
                     reason = "Unknown SSL protocol connecting to host '%s' for "\
                          "repository '%s'. OpenSSL responded:\n%s"\
                          % (self.hostname, self.repos, e)
-                raise OfflineImapError(reason, severity)
+                raise OfflineImapError(reason, severity), None, exc_info()[2]
 
             elif isinstance(e, socket.error) and e.args[0] == errno.ECONNREFUSED:
                 # "Connection refused", can be a non-existing port, or an unauthorized
@@ -489,27 +567,29 @@ class IMAPServer:
                 reason = "Connection to host '%s:%d' for repository '%s' was "\
                     "refused. Make sure you have the right host and port "\
                     "configured and that you are actually able to access the "\
-                    "network." % (self.hostname, self.port, self.repos)
-                raise OfflineImapError(reason, severity)
+                    "network."% (self.hostname, self.port, self.repos)
+                raise OfflineImapError(reason, severity), None, exc_info()[2]
             # Could not acquire connection to the remote;
             # socket.error(last_error) raised
             if str(e)[:24] == "can't open socket; error":
                 raise OfflineImapError("Could not connect to remote server '%s' "\
                     "for repository '%s'. Remote does not answer."
                     % (self.hostname, self.repos),
-                    OfflineImapError.ERROR.REPO)
+                    OfflineImapError.ERROR.REPO), None, exc_info()[2]
             else:
                 # re-raise all other errors
                 raise
 
     def connectionwait(self):
-        """Waits until there is a connection available.  Note that between
-        the time that a connection becomes available and the time it is
-        requested, another thread may have grabbed it.  This function is
-        mainly present as a way to avoid spawning thousands of threads
-        to copy messages, then have them all wait for 3 available connections.
-        It's OK if we have maxconnections + 1 or 2 threads, which is what
-        this will help us do."""
+        """Waits until there is a connection available.
+
+        Note that between the time that a connection becomes available and the
+        time it is requested, another thread may have grabbed it.  This function
+        is mainly present as a way to avoid spawning thousands of threads to
+        copy messages, then have them all wait for 3 available connections.
+        It's OK if we have maxconnections + 1 or 2 threads, which is what this
+        will help us do."""
+
         self.semaphore.acquire()
         self.semaphore.release()
 
@@ -533,11 +613,13 @@ class IMAPServer:
             self.gssapi = False
 
     def keepalive(self, timeout, event):
-        """Sends a NOOP to each connection recorded.   It will wait a maximum
-        of timeout seconds between doing this, and will continue to do so
-        until the Event object as passed is true.  This method is expected
-        to be invoked in a separate thread, which should be join()'d after
-        the event is set."""
+        """Sends a NOOP to each connection recorded.
+
+        It will wait a maximum of timeout seconds between doing this, and will
+        continue to do so until the Event object as passed is true.  This method
+        is expected to be invoked in a separate thread, which should be join()'d
+        after the event is set."""
+
         self.ui.debug('imap', 'keepalive thread started')
         while not event.isSet():
             self.connectionlock.acquire()
@@ -547,7 +629,7 @@ class IMAPServer:
 
             threads = []
             for i in range(numconnections):
-                self.ui.debug('imap', 'keepalive: processing connection %d of %d' % (i, numconnections))
+                self.ui.debug('imap', 'keepalive: processing connection %d of %d'% (i, numconnections))
                 if len(self.idlefolders) > i:
                     # IDLE thread
                     idler = IdleThread(self, self.idlefolders[i])
@@ -570,14 +652,14 @@ class IMAPServer:
         return
 
     def __verifycert(self, cert, hostname):
-        '''Verify that cert (in socket.getpeercert() format) matches hostname.
+        """Verify that cert (in socket.getpeercert() format) matches hostname.
+
         CRLs are not handled.
+        Returns error message if any problems are found and None on success."""
 
-        Returns error message if any problems are found and None on success.
-        '''
         errstr = "CA Cert verifying failed: "
         if not cert:
-            return ('%s no certificate received' % errstr)
+            return ('%s no certificate received'% errstr)
         dnsname = hostname.lower()
         certnames = []
 
@@ -585,7 +667,7 @@ class IMAPServer:
         notafter = cert.get('notAfter')
         if notafter:
             if time.time() >= cert_time_to_seconds(notafter):
-                return '%s certificate expired %s' % (errstr, notafter)
+                return '%s certificate expired %s'% (errstr, notafter)
 
         # First read commonName
         for s in cert.get('subject', []):
@@ -593,7 +675,7 @@ class IMAPServer:
             if key == 'commonName':
                 certnames.append(value.lower())
         if len(certnames) == 0:
-            return ('%s no commonName found in certificate' % errstr)
+            return ('%s no commonName found in certificate'% errstr)
 
         # Then read subjectAltName
         for key, value in cert.get('subjectAltName', []):
@@ -606,7 +688,7 @@ class IMAPServer:
                 '.' in dnsname and certname == '*.' + dnsname.split('.', 1)[1]):
                 return None
 
-        return ('%s no matching domain name found in certificate' % errstr)
+        return ('%s no matching domain name found in certificate'% errstr)
 
 
 class IdleThread(object):
@@ -614,6 +696,7 @@ class IdleThread(object):
         """If invoked without 'folder', perform a NOOP and wait for
         self.stop() to be called. If invoked with folder, switch to IDLE
         mode and synchronize once we have a new message"""
+
         self.parent = parent
         self.folder = folder
         self.stop_sig = Event()
@@ -634,18 +717,18 @@ class IdleThread(object):
         self.thread.join()
 
     def noop(self):
-        #TODO: AFAIK this is not optimal, we will send a NOOP on one
-        #random connection (ie not enough to keep all connections
-        #open). In case we do the noop multiple times, we can well use
-        #the same connection every time, as we get a random one. This
-        #function should IMHO send a noop on ALL available connections
-        #to the server.
+        # TODO: AFAIK this is not optimal, we will send a NOOP on one
+        # random connection (ie not enough to keep all connections
+        # open). In case we do the noop multiple times, we can well use
+        # the same connection every time, as we get a random one. This
+        # function should IMHO send a noop on ALL available connections
+        # to the server.
         imapobj = self.parent.acquireconnection()
         try:
             imapobj.noop()
         except imapobj.abort:
-            self.ui.warn('Attempting NOOP on dropped connection %s' % \
-                             imapobj.identifier)
+            self.ui.warn('Attempting NOOP on dropped connection %s'%
+                imapobj.identifier)
             self.parent.releaseconnection(imapobj, True)
             imapobj = None
         finally:
@@ -671,9 +754,10 @@ class IdleThread(object):
         ui.unregisterthread(currentThread()) #syncfolder registered the thread
 
     def __idle(self):
-        """Invoke IDLE mode until timeout or self.stop() is invoked"""
+        """Invoke IDLE mode until timeout or self.stop() is invoked."""
+
         def callback(args):
-            """IDLE callback function invoked by imaplib2
+            """IDLE callback function invoked by imaplib2.
 
             This is invoked when a) The IMAP server tells us something
             while in IDLE mode, b) we get an Exception (e.g. on dropped
@@ -683,44 +767,48 @@ class IdleThread(object):
             if exc_data is None and not self.stop_sig.isSet():
                 # No Exception, and we are not supposed to stop:
                 self.needsync = True
-            self.stop_sig.set() # continue to sync
+            self.stop_sig.set() # Continue to sync.
 
         while not self.stop_sig.isSet():
             self.needsync = False
 
-            success = False # successfully selected FOLDER?
+            success = False # Successfully selected FOLDER?
             while not success:
                 imapobj = self.parent.acquireconnection()
                 try:
                     imapobj.select(self.folder)
                 except OfflineImapError as e:
                     if e.severity == OfflineImapError.ERROR.FOLDER_RETRY:
-                        # Connection closed, release connection and retry
+                        # Connection closed, release connection and retry.
                         self.ui.error(e, exc_info()[2])
                         self.parent.releaseconnection(imapobj, True)
+                    elif e.severity == OfflineImapError.ERROR.FOLDER:
+                        # Just continue the process on such error for now.
+                        self.ui.error(e, exc_info()[2])
                     else:
-                        raise e
+                        # Stops future attempts to sync this account.
+                        raise
                 else:
                     success = True
             if "IDLE" in imapobj.capabilities:
                 imapobj.idle(callback=callback)
             else:
                 self.ui.warn("IMAP IDLE not supported on server '%s'."
-                    "Sleep until next refresh cycle." % imapobj.identifier)
+                    "Sleep until next refresh cycle."% imapobj.identifier)
                 imapobj.noop()
-            self.stop_sig.wait() # self.stop() or IDLE callback are invoked
+            self.stop_sig.wait() # self.stop() or IDLE callback are invoked.
             try:
                 # End IDLE mode with noop, imapobj can point to a dropped conn.
                 imapobj.noop()
             except imapobj.abort:
-                self.ui.warn('Attempting NOOP on dropped connection %s' % \
-                                 imapobj.identifier)
+                self.ui.warn('Attempting NOOP on dropped connection %s'%
+                    imapobj.identifier)
                 self.parent.releaseconnection(imapobj, True)
             else:
                 self.parent.releaseconnection(imapobj)
 
             if self.needsync:
-                # here not via self.stop, but because IDLE responded. Do
+                # Here not via self.stop, but because IDLE responded. Do
                 # another round and invoke actual syncing.
                 self.stop_sig.clear()
                 self.__dosync()
